#include <Wire.h> //This library allows you to communicate with I2C / TWI devices.
#include <WiFi.h>
#include <SPI.h>
#include<WString.h>

char ssid[] = "SSID";
char pass[] = "Password";

int status = WL_IDLE_STATUS;

WiFiClient client;
IPAddress server(130,206,83,2); 
String line="";

int red_led=8;
int green_led=9;

int reading = 0;
boolean detected;
int car_in=0;
int car_out=0;
int lots=0; //Occuped lots in parking
int max_car=3;//number of available lots

void setup()
{
  Wire.begin();// join i2c bus for using HC SR501
  Serial.begin(9600);

  while (status != WL_CONNECTED) { 
    status = WiFi.begin(ssid, pass);
    delay(10000); 
  } 
  Serial.println("Wifi OK");

  pinMode(red_led, OUTPUT);
  pinMode(green_led, OUTPUT);

  detected =false;  // start serial communication at 9600bps
  attachInterrupt(0,trig, RISING);// 0-->pin2 1--Pin3
  delay(1000);
  updateContext(car_in);
  updateLed(lots);
}



void loop(){
  while(lots>=max_car){ //If parking is full, wait until some car get out
    car_out=queryContext();
    lots=car_in-car_out;
    delay(15000);
  }
  updateLed(lots);
  while(detected){
    reading=getDistance();        
    Serial.println(reading);   // print the reading
    if(reading < 100){
      detected=false;
      attachInterrupt(0,trig,RISING);
      Serial.println("There's a new car");
      car_in++;
      car_out=queryContext();
      lots=car_in-car_out;
      delay(2000);
      updateContext(car_in);
      Serial.print("CarsIn: ");
      Serial.println(car_in);
      Serial.print("CarsOut: ");
      Serial.println(car_out);
      Serial.print("Occupied: ");
      Serial.println(lots);
      
      updateLed(lots);
    }
    delay(250);
  }

}

//FUNCTIONS

void trig(){
  if(!detected){
    detected=true;
    detachInterrupt(0);
  } 
}

int getDistance(){
  int dist=0;
  // step 1: instruct sensor to read echoes
  Wire.beginTransmission(112); 

  Wire.write(byte(0x00));       
  Wire.write(byte(0x51));       
  Wire.endTransmission();     
  //step 2: wait for readings to happen
  delay(70);                  
  // step 3: instruct sensor to return a particular echo reading
  Wire.beginTransmission(112); // transmit to device #112
  Wire.write(byte(0x02));      // sets register pointer to echo #1 register (0x02)
  Wire.endTransmission();      // stop transmitting
  // step 4: request reading from sensor
  Wire.requestFrom(112, 2);    // request 2 bytes from slave device #112
  // step 5: receive reading from sensor
  if(2 <= Wire.available()){   // if two bytes were received
    dist = Wire.read();  // receive high byte (overwrites previous reading)
    dist= dist << 8;    // shift high byte to be high 8 bits
    dist |= Wire.read(); // receive low byte as lower 8 bits
  }
  return dist;
}

void updateLed(int val){
  if(val<max_car){
    digitalWrite(red_led, LOW);
    digitalWrite(green_led, HIGH);
  }
  if(val>=max_car){
    digitalWrite(red_led, HIGH);
    digitalWrite(green_led, LOW);  
  } 
}

int queryContext(){
  int val=0;
  if (client.connect(server, 1026)) {
    Serial.println("Query context");
    client.println("POST /NGSI10/queryContext HTTP/1.1");
    client.println("Host: 130.206.83.2:1026");
    client.println("User-Agent: Arduino/1.1");
    client.println("Connection: close");
    client.println("Content-Type: application/xml");
    client.print("Content-Length: ");
    client.println("274");
    client.println();
    client.println("<?xml version=\"1.0\" encoding=\"UFT-8\"?>");
    client.println("<queryContextRequest>");
    client.println("<entityIdList>");
    client.println("<entityId type=\"UPCT:PARKING\" isPattern=\"false\">");
    client.println("<id>UPCT:PARKING:ED</id>");
    client.println("</entityId>");
    client.println("</entityIdList>");
    client.println("<attributeList>");
    client.println("<attribute>car_out</attribute>");
    client.println("</attributeList>");
    //client.println("<attributeList/>");
    client.println("</queryContextRequest>");
    client.println();

    Serial.println("Done");
    //delay(1000);
    while(!client.available());//Using this while we wait for and reply but it can have problems
  }                            //It's possible to use delay(750); instead,
  else{
    Serial.println("ERROR");
  }
  
  while (client.available()) {
    char c = client.read();
    //Serial.write(c);
    line=line + c;
    if(c==10){
      val=searchValue(line,val);
      line="";
    }
    
  }

    Serial.println("Finished...");
    client.stop();
  
  return val;
}

void updateContext(int val){

  if (client.connect(server, 1026)) {
    Serial.println("Update context");
    client.println("POST /NGSI10/updateContext HTTP/1.1");
    client.println("Host: 130.206.83.2:1026");
    client.println("User-Agent: Arduino/1.1");
    client.println("Connection: close");
    client.println("Content-Type: application/xml");
    client.print("Content-Length: ");
    client.println("460");
    client.println();
    client.println("<?xml version=\"1.0\" encoding=\"UFT-8\"?>");
    client.println("<updateContextRequest>");
    client.println("<contextElementList>");
    client.println("<contextElement>");
    client.println("<entityId type=\"UPCT:PARKING\" isPattern=\"false\">");
    client.println("<id>UPCT:PARKING:ED</id>");
    client.println("</entityId>");
    client.println("<contextAttributeList>");
    client.println("<contextAttribute>");
    client.println("<name>car_in</name>");
    client.println("<type>lot</type>");
    client.print("<contextValue>");
    client.print(val);
    client.println("</contextValue>");
    client.println("</contextAttribute>");
    client.println("</contextAttributeList>");
    client.println("</contextElement>");
    client.println("</contextElementList>");
    client.println("<updateAction>UPDATE</updateAction>");
    client.println("</updateContextRequest>");
    client.println();

    Serial.println("Done");
    //delay(1000);
    while(!client.available());//Using this while we wait for and reply but it can have problems
                            //It's possible to use delay(750); instead,
  }
  else{
    Serial.println("ERROR");
  }

  
  //Use when printing Http response is required
  //while (client.available()) {
 //char c = client.read();
  //Serial.write(c);           
  //}
  
 
    Serial.println("Finished...");
    client.stop();
 
}

int searchValue(String s,int i) {
  int beginning,ending;
  String val;
  beginning=s.indexOf('>');
  ending=s.indexOf('<', beginning+1);
  if(s.startsWith("contextValue",s.indexOf('<')+1)){
    val =s.substring(beginning+1,ending);
    return val.toInt();
  }
  else{
    return i;
  }
}


